{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 310,
   "id": "58fb5138-40e2-45e5-94d4-602e58165fa6",
   "metadata": {},
   "outputs": [],
   "source": [
    "# python -m pip install --index-url=https://bcms.bloomberg.com/pip/simple blpapi\n",
    "# python -m pip install xbbg\n",
    "# python -m pip install numpy\n",
    "# python -m pip install scikit-learn\n",
    "\n",
    "from xbbg import blp\n",
    "import numpy as np\n",
    "from sklearn.linear_model import LinearRegression\n",
    "import math"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 311,
   "id": "866a94d3-b953-4e40-9df3-93ebaa7168e5",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Time_Series:\n",
    "    \n",
    "    def __init__(self, s1, s2, start, end, fld='TOT_RETURN_INDEX_GROSS_DVDS'):\n",
    "        self._s1 = s1\n",
    "        self._s2 = s2\n",
    "        self._fld= fld\n",
    "\n",
    "        self._df = blp.bdh(tickers=[self._s1, self._s2], flds=self._fld, start_date=start, end_date=end,)\n",
    "        \n",
    "        self._df.columns = [self._s1, self._s2]\n",
    "        self._df = self._df.dropna()\n",
    "        \n",
    "        self._df['r1'] = np.log(self._df[self._s1] / self._df[self._s1].shift(1))\n",
    "        self._df['r2'] = np.log(self._df[self._s2] / self._df[self._s2].shift(1))\n",
    "        \n",
    "        self._df = self._df.dropna()\n",
    "        \n",
    "        c = self._df.columns.tolist()\n",
    "        c = [c[0], c[2], c[1], c[3]]\n",
    "        self._df = self._df[c]\n",
    "        \n",
    "        self._n = len(self._df)\n",
    "        \n",
    "        X = self._df['r2'].values.reshape(-1, 1)\n",
    "        y = self._df['r1'].values\n",
    "        model = LinearRegression().fit(X, y)\n",
    "        self._ols_a   = model.intercept_\n",
    "        self._ols_b   = model.coef_\n",
    "        self._ols_rsq = model.score(X, y)        \n",
    "        \n",
    "        return None\n",
    "    \n",
    "    def df(self):\n",
    "        return self._df\n",
    "        \n",
    "    def time_series_return(self, s):\n",
    "        return np.mean(s)\n",
    "    \n",
    "    def time_series_std_dev(self, s):\n",
    "        return np.std(s)\n",
    "    \n",
    "    def annualized_return(self, s):\n",
    "        return np.mean(s) * 252\n",
    "    \n",
    "    def annualized_std_dev(self, s):\n",
    "        return np.std(s) * math.sqrt(252)\n",
    "    \n",
    "    def analysis(self, analysis_type='e'):\n",
    "        \n",
    "        t = list(analysis_type)[0].lower()\n",
    "        \n",
    "        d = {\n",
    "            'e' : self._df['r1'],\n",
    "            'b' : self._df['r2'],\n",
    "            'a' : self._df['r1'] - self._df['r2'],\n",
    "        }\n",
    "        \n",
    "        s = d.get(t, self._df['r1'])\n",
    "        \n",
    "        r_ts = self.time_series_return(s)\n",
    "        r_an = self.annualized_return(s)\n",
    "        \n",
    "        sd_ts = self.time_series_std_dev(s)\n",
    "        sd_an = self.annualized_std_dev(s)\n",
    "        \n",
    "        a = {\n",
    "            'e' : self._s1,\n",
    "            'b' : self._s2,\n",
    "            'a' : 'Active',\n",
    "        }\n",
    "        \n",
    "        print(a.get(t, self._s1) + ' Analysis:')\n",
    "        print(\"%30s %15s\" % ('Time Series', 'Annualized'))\n",
    "        print(\"Total return   %15.6f\" % (r_ts * self._n))\n",
    "        print(\"Num. obs.      %15d\" % self._n)\n",
    "        print(\"Mean return    %15.6f %15.6f\" % (r_ts, r_an))\n",
    "        print(\"Std Devn       %15.6f %15.6f\" % (sd_ts, sd_an))\n",
    "        print(\"Ratio          %15.6f %15.6f\" % (r_ts/sd_ts, r_an/sd_an))\n",
    "        if t == 'e':\n",
    "            print(\"OLS Intercept  %15.6f\" % self._ols_a)\n",
    "            print(\"OLS Beta       %15.6f\" % self._ols_b)\n",
    "            print(\"OLS R-squared  %15.6f\" % self._ols_rsq)\n",
    "        \n",
    "        return None\n",
    "        \n",
    "    def equity_analysis(self):\n",
    "        return self.analysis('e')\n",
    "    \n",
    "    def benchmark_analysis(self):\n",
    "        return self.analysis('e')\n",
    "    \n",
    "    def active_analysis(self):\n",
    "        return self.analysis('a')\n",
    "        "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 312,
   "id": "3e29da0a-3751-4622-b5ed-6f8e32c43113",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>JABAX US Equity</th>\n",
       "      <th>r1</th>\n",
       "      <th>MSAAMMOR Index</th>\n",
       "      <th>r2</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2021-01-04</th>\n",
       "      <td>40.55</td>\n",
       "      <td>-0.010548</td>\n",
       "      <td>4220.494</td>\n",
       "      <td>-0.003589</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2021-01-05</th>\n",
       "      <td>40.65</td>\n",
       "      <td>0.002463</td>\n",
       "      <td>4232.586</td>\n",
       "      <td>0.002861</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2021-01-06</th>\n",
       "      <td>40.63</td>\n",
       "      <td>-0.000492</td>\n",
       "      <td>4251.219</td>\n",
       "      <td>0.004393</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2021-01-07</th>\n",
       "      <td>40.99</td>\n",
       "      <td>0.008821</td>\n",
       "      <td>4274.727</td>\n",
       "      <td>0.005514</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2021-01-08</th>\n",
       "      <td>41.11</td>\n",
       "      <td>0.002923</td>\n",
       "      <td>4289.767</td>\n",
       "      <td>0.003512</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "            JABAX US Equity        r1  MSAAMMOR Index        r2\n",
       "2021-01-04            40.55 -0.010548        4220.494 -0.003589\n",
       "2021-01-05            40.65  0.002463        4232.586  0.002861\n",
       "2021-01-06            40.63 -0.000492        4251.219  0.004393\n",
       "2021-01-07            40.99  0.008821        4274.727  0.005514\n",
       "2021-01-08            41.11  0.002923        4289.767  0.003512"
      ]
     },
     "execution_count": 312,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ts = Time_Series('JABAX US Equity', 'MSAAMMOR Index', '2020-12-31', '2021-06-30') # 'SOFRINDX Index'\n",
    "ts.df().head()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 313,
   "id": "0b201282-2d08-4e0d-bc1a-7aee8854c2a8",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "JABAX US Equity Analysis:\n",
      "                   Time Series      Annualized\n",
      "Total return          0.081506\n",
      "Num. obs.                  124\n",
      "Mean return           0.000657        0.165641\n",
      "Std Devn              0.005933        0.094183\n",
      "Ratio                 0.110788        1.758711\n",
      "OLS Intercept         0.000001\n",
      "OLS Beta              1.161297\n",
      "OLS R-squared         0.715799\n"
     ]
    }
   ],
   "source": [
    "ts.analysis()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 314,
   "id": "af68ff5c-0d72-4fb9-8730-30c698f5c5d4",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "MSAAMMOR Index Analysis:\n",
      "                   Time Series      Annualized\n",
      "Total return          0.070063\n",
      "Num. obs.                  124\n",
      "Mean return           0.000565        0.142385\n",
      "Std Devn              0.004322        0.068616\n",
      "Ratio                 0.130719        2.075107\n"
     ]
    }
   ],
   "source": [
    "ts.analysis('b')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 315,
   "id": "818f7e1a-076e-4af4-ac15-3095e16072f2",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Active Analysis:\n",
      "                   Time Series      Annualized\n",
      "Total return          0.011443\n",
      "Num. obs.                  124\n",
      "Mean return           0.000092        0.023255\n",
      "Std Devn              0.003239        0.051415\n",
      "Ratio                 0.028493        0.452308\n"
     ]
    }
   ],
   "source": [
    "ts.active_analysis()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 316,
   "id": "f8541df1-8a66-48f6-bbc2-519208c7ac0a",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Active Analysis:\n",
      "                   Time Series      Annualized\n",
      "Total return          0.081362\n",
      "Num. obs.                  124\n",
      "Mean return           0.000656        0.165348\n",
      "Std Devn              0.005933        0.094183\n",
      "Ratio                 0.110593        1.755608\n"
     ]
    }
   ],
   "source": [
    "ts_rf = Time_Series('JABAX US Equity', 'SOFRINDX Index', '2020-12-31', '2021-06-30')\n",
    "ts_rf.analysis('ACTV')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "232417ff-c970-4f0e-8c00-2ecef065557b",
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.0"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
